apiVersion: v1
kind: Config
current-context: scheduler-to-{{ cluster_name }}
preferences: {}
clusters:
- cluster:
    certificate-authority: /etc/kubernetes/certs/k8sdc-ca.crt
    server: https://{{ groups['k8sdc-masters'][0] }}:{{ kubernetes_api_port }}
  name: {{ cluster_name }}
contexts:
- context:
    cluster: {{ cluster_name }}
    user: scheduler
  name: scheduler-to-{{ cluster_name }}
users:
- name: scheduler
  user:
    token: {{ master_token_values[inventory_hostname]['system_scheduler'] | b64decode }}
